@@ -2,6 +2,7 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
+#import pysnooper
 import copy
 import inspect
 import os
@@ -16,14 +17,14 @@ import json
 from typing import List
 
 import numpy as np
-from trident.backend.common import to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session, \
+from trident.backend.common import to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session,get_backend, \
     snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict, split_path, sanitize_path,make_dir_if_need,Signature
 from trident.backend.tensorspec import *
 from trident.data.image_common import *
 from trident.callbacks import LambdaCallback, UnfreezeModelCallback
 
 _session = get_session()
-_backend = _session.backend
+_backend = get_backend()
 if _backend == 'pytorch':
     from trident.backend.pytorch_backend import *
     from trident.backend.pytorch_ops import *
@@ -52,7 +53,7 @@ begin_time = last_time
 class ModelBase(object):
     def __init__(self, inputs=None,  input_shape=None,output=None, name='', **kwargs):
         if isinstance(inputs,tuple) and isinstance(inputs[0],int):
-            input_shape.inputs=inputs,input_shape
+            input_shape,inputs=inputs,input_shape
         self.batch_index = 0
         self.filter_index = 1
         self.inputs = OrderedDict()
@@ -149,7 +150,11 @@ class ModelBase(object):
 
     @property
     def outputs(self):
-        return self._outputs
+        if self._model is not None and isinstance(self._model ,Layer):
+            if len(self._outputs)==1 and is_tensor(self._model.output_shape)  and not assert_input_compatibility(self._outputs.value_list[0],self._model.output_shape):
+                self._outputs[self._outputs.key_list[0]]=TensorSpec(shape=self._model.output_shape,name=self._outputs.key_list[0])
+
+            return self._outputs
 
     @outputs.setter
     def outputs(self, value):
@@ -219,9 +224,9 @@ class ModelBase(object):
 
 
     def update_signature(self, arg_names):
-        if self.model is not None and hasattr(self.model, 'signature'):
-            self._signature = self.model.signature
-        if self._signature is None or len(self._signature.inputs.key_list)+len(self._signature.outputs.key_list) == len(arg_names):
+        if self.model is not None and hasattr(self.model, 'signature') and self.signature != self.model.signature:
+            self.signature = self.model.signature
+        if self.signature is None or len(self.signature.inputs.key_list)+len(self.signature.outputs.key_list) == len(arg_names):
 
             new_inputs =  OrderedDict()
             for i in range(len(arg_names[:len(self.inputs)])):
@@ -243,16 +248,15 @@ class ModelBase(object):
                 new_target[target_arg] = targets.value_list[0]
             self._outputs = new_outputs
             self._targets = new_target
-            if self.model is not None:
-                self._signature = get_signature(self._model.forward, 'model')
-                self._signature.inputs = copy.deepcopy(self.inputs)
-                self._signature.outputs = copy.deepcopy(self._outputs)
-                self.model.signature=self._signature
+            # if self.model is not None:
+            #     self.signature = get_signature(self._model.forward, 'model')
+            #     self.signature.inputs = copy.deepcopy(self.inputs)
+            #     self.signature.outputs = copy.deepcopy(self._outputs)
 
-            print(self._model.signature)
+            print(self.signature)
         elif not isinstance(arg_names, (list, tuple)):
             raise ValueError('arg_names should be list or tuple')
-        elif len(self._signature) != len(arg_names):
+        elif len(self.signature) != len(arg_names):
             raise ValueError('data deed and arg_names should be the same length')
 
 
@@ -293,7 +297,7 @@ class ModelBase(object):
             img_data = reverse_image_backend_adaption(img_data)
         return img_data
 
-    def _initial_graph(self, inputs=None, output=None, input_shape=None):
+    def _initial_graph(self, inputs=None, input_shape=None, output=None):
         pass
 
     def complie(self,optimizer="Adam",
@@ -329,6 +333,10 @@ class ModelBase(object):
 
 
     def __getattr__(self, name):
+        if name == 'signature' or name == '_signature':
+            _model = self.__dict__['_model']
+            if _model is not None and isinstance(_model, Layer):
+                return _model.signature
         if 'training_context' in self.__dict__:
             if name in  self.__dict__['training_context']:
                 return  self.__dict__['training_context'][name]
@@ -352,6 +360,10 @@ class ModelBase(object):
         raise AttributeError("'{}' object has no attribute '{}'".format(type(self).__name__, name))
 
     def __setattr__(self, name, value):
+        if name=='signature' or name=='_signature':
+            _model = self.__dict__['_model']
+            if _model is not None and isinstance(_model ,Layer):
+                object.__setattr__(_model, "_" + 'signature', value)
         if 'training_context' in self.__dict__ and name in self.__dict__['training_context']:
             self.__dict__['training_context'][name]=value
         elif '_model' in self.__dict__ and self.__dict__['_model']  :
@@ -521,10 +533,10 @@ class ModelBase(object):
 
     def do_post_gradient_update(self):
 
-        self.training_context['tmp_losses'].collect('total_losses',self.training_context['steps'],to_numpy(self.training_context['current_loss']).mean())
+        self.training_context['tmp_losses'].collect('total_losses',self.training_context['steps'],self.training_context['current_loss'])
         if self.training_context['is_collect_data'] == True:
             steps,values=self.training_context['tmp_losses'].get_series('total_losses')
-            self.training_context['losses'].collect('total_losses',self.training_context['steps'],float(to_numpy(values).mean()))
+            self.training_context['losses'].collect('total_losses',self.training_context['steps'],float(np.asarray(values).mean()))
             self.training_context['tmp_losses'].reset()
 
     def do_on_metrics_evaluation_start(self):
@@ -600,6 +612,8 @@ class ModelBase(object):
         raise NotImplementedError
 
     def print_batch_progress(self, print_batch_progress_frequency):
+        if 'max_name_length' not in self.training_context:
+            self.training_context['max_name_length']=len(self.name)+1
         metric_strings=[]
         for k, v in self._metrics.items():
             collect_history = self._metrics[k].collect_history
@@ -627,10 +641,11 @@ class ModelBase(object):
         self.training_context['time_batch_progress'] = progress_end
         progress_bar(step_time,self.training_context['current_batch'], self.training_context['total_batch'],
                  'Loss: {0:<8.5f}| {1} | learning rate: {2:<10.3e}| epoch: {3}'.format(loss_value, ','.join(metric_strings), self.training_context['current_lr'],
-                     self.training_context['current_epoch']), name=self.name)
+                     self.training_context['current_epoch']), name=self.name.ljust(self.training_context['max_name_length']+1,' '))
 
     def print_epoch_progress(self, print_epoch_progress_frequency):
-
+        if 'max_name_length' not in self.training_context:
+            self.training_context['max_name_length']=len(self.name)+1
         metric_strings=[]
         loss_steps, loss_values = self.epoch_loss_history.get_series('total_losses')
         if print_epoch_progress_frequency>len(loss_values):
@@ -652,11 +667,12 @@ class ModelBase(object):
         step_time = progress_end-progress_start
         self.training_context['time_epoch_progress']=progress_end
         progress_bar(step_time,self.training_context['current_epoch'], self.training_context['total_epoch'],
-                     'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.3e}'.format(loss_value, ','.join(metric_strings), self.training_context['current_lr']), name=self.name)
+                     'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.3e}'.format(loss_value, ','.join(metric_strings), self.training_context['current_lr']), name=self.name.ljust(self.training_context['max_name_length']+1,' '))
 
+    #@pysnooper.snoop()
     def train_model(self, train_data, test_data, current_epoch, current_batch, total_epoch, total_batch,
                     is_collect_data=True, is_print_batch_progress=True, is_print_epoch_progress=True,
-                    is_print_batch_gradients=True, log_gradients=False, log_weights=False, accumulate_grads=False):
+                    is_print_batch_gradients=True, log_gradients=False, log_weights=False, accumulate_grads=False,is_out_sample_evaluation=False,**kwargs):
         try:
             self.training_context['current_epoch'] = current_epoch
             self.training_context['current_batch'] = current_batch
@@ -717,7 +733,9 @@ class ModelBase(object):
                     if isinstance(output, (list, tuple)):
                         for i in range(len(output)):
                             train_data[self.outputs.key_list[i]] = output[i]
-
+                    elif isinstance(output, (OrderedDict)):
+                        for k,v in output.items():
+                            train_data[k] = v
                     elif 'tensor' in output.__class__.__name__.lower():
                         train_data[self.outputs.key_list[0]] = output
                         if self.use_output_as_loss==True:
@@ -770,7 +788,7 @@ class ModelBase(object):
                             self.training_context['current_loss'] =self.training_context['current_loss']+ overall_loss
 
                             if is_collect_data:
-                                self.training_context['losses'].collect(k,self.training_context['steps'],float(to_numpy(overall_loss)))
+                                self.training_context['losses'].collect(k,self.training_context['steps'],overall_loss)
 
                         else:
                             if any_abnormal_number(this_loss):
@@ -779,7 +797,7 @@ class ModelBase(object):
                                 #a leaf Variable that requires grad connotused in an in-place operation.
                                 self.training_context['current_loss'] =self.training_context['current_loss'] + this_loss
                             if is_collect_data:
-                                self.training_context['losses'].collect(k, self.training_context['steps'], float(to_numpy(this_loss)))
+                                self.training_context['losses'].collect(k, self.training_context['steps'], this_loss)
                     except Exception as e:
                         print(e)
                         PrintException()
@@ -802,7 +820,7 @@ class ModelBase(object):
                         self.training_context['current_loss'] =self.training_context['current_loss'] + this_loss  # self.training_context[
                     # 'current_loss'] + this_loss
                     if is_collect_data:
-                        self.training_context['losses'].collect(k + '_Loss', self.training_context['steps'], float(to_numpy(this_loss)))
+                        self.training_context['losses'].collect(k + '_Loss', self.training_context['steps'], this_loss)
 
 
 
@@ -837,7 +855,7 @@ class ModelBase(object):
 
 
 
-                if test_data is not None and len(test_data) > 0 and  self.training_context['stop_update']<1 :
+                if is_out_sample_evaluation==True and test_data is not None and len(test_data) > 0 and  self.training_context['stop_update']<1 :
                     tmp_output = try_map_args_and_call(self._model, test_data, self.training_context['data_feed'])
                     if isinstance(tmp_output, (list, tuple)):
                         for i in range(len(tmp_output)):
@@ -863,12 +881,12 @@ class ModelBase(object):
                         self.training_context['tmp_metrics'].regist(k)
 
                     this_metric = try_map_args_and_call(v, train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else to_tensor(0)
-                    self.training_context['tmp_metrics'].collect(k, self.training_context['steps'], float(to_numpy(this_metric)))
+                    self.training_context['tmp_metrics'].collect(k, self.training_context['steps'], this_metric)
 
 
-                    if test_data is not None and len(test_data) > 0 and collect_history!=False :
+                    if is_out_sample_evaluation==True and test_data is not None and len(test_data) > 0 and collect_history!=False :
                         this_out_metric = try_map_args_and_call(v, test_data , self.training_context['data_feed'])
-                        self.training_context['out_sample_metrics'].collect(k, self.training_context['steps'], float(to_numpy(this_out_metric)))
+                        self.training_context['out_sample_metrics'].collect(k, self.training_context['steps'], this_out_metric)
 
                 # ON_EVALUATION_END
                 self.do_on_metrics_evaluation_end()
@@ -882,7 +900,7 @@ class ModelBase(object):
                     #aggregate tmp data and move to metrics history
                     for k, v in self.training_context['tmp_metrics'].items():
                         steps,values=self.training_context['tmp_metrics'].get_series(k)
-                        self.training_context['metrics'].collect(k, self.training_context['steps'], float(to_numpy(values).mean()))
+                        self.training_context['metrics'].collect(k, self.training_context['steps'], np.asarray(values).mean())
                     self.training_context['tmp_metrics'].reset()
 
                 # ON_BATCH_END
@@ -905,20 +923,29 @@ class ModelBase(object):
                 else:
                     self.training_context['print_batch_progress_frequency'] += 1
 
-                if test_data is not None and len(test_data) > 0:
-                    print(self.training_context['model_name']+': out-of-sample evaluation: ',','.join(['{0}: {1:<8.3%}'.format(k, v[-1][-1]) for k, v in self.training_context['out_sample_metrics'].items()]))
-
-
+                if is_out_sample_evaluation==True and test_data is not None and len(test_data) > 0:
+                    verbose=[]
+                    for k in self.training_context['out_sample_metrics'].get_keys():
+                        test_steps, test_values = self.training_context['out_sample_metrics'].get_series(k)
+                        metric_value= test_values[-1]
+                        history_metric_value=np.array(test_values).mean()
 
+                        format_string = '.3%'
+                        if history_metric_value > 3:
+                            format_string = '.3f'
+                        elif history_metric_value < 1e-3:
+                            format_string = '.3e'
+                        verbose.append('{0}: {1:<8{2}}'.format(k, metric_value, format_string))
+                    print(self.training_context['model_name'] + ': out-of-sample evaluation: ',','.join(verbose))
 
             if self.training_context['current_batch'] == self.training_context['total_batch'] - 1:
                 self.do_on_epoch_end()
                 batch_steps,batch_values=self.training_context['losses'].get_series('total_losses')
                 if not hasattr(self.training_context['losses'],'last_aggregate_idx'):
-                    self.epoch_loss_history.collect('total_losses',self.training_context['current_epoch'],np.array(batch_values).mean())
+                    self.epoch_loss_history.collect('total_losses',self.training_context['current_epoch'],np.asarray(batch_values).mean())
                     self.training_context['losses'].last_aggregate_idx=len(batch_values)
                 else:
-                    self.epoch_loss_history.collect('total_losses', self.training_context['current_epoch'], np.array(batch_values[self.training_context['losses'].last_aggregate_idx:]).mean())
+                    self.epoch_loss_history.collect('total_losses', self.training_context['current_epoch'], np.asarray(batch_values[self.training_context['losses'].last_aggregate_idx:]).mean())
                     self.training_context['losses'].last_aggregate_idx = len(batch_values)
 
 
@@ -926,10 +953,10 @@ class ModelBase(object):
                 for k, v in self.training_context['metrics'].items():
                     metric_steps, metric_values = self.training_context['metrics'].get_series(k)
                     if not hasattr(self.training_context['metrics'], 'last_aggregate_idx'):
-                        self.epoch_metric_history.collect(k, self.training_context['current_epoch'], np.array(metric_values).mean())
+                        self.epoch_metric_history.collect(k, self.training_context['current_epoch'], np.asarray(metric_values).mean())
                         self.training_context['metrics'].last_aggregate_idx = len(metric_values)
                     else:
-                        self.epoch_metric_history.collect(k, self.training_context['current_epoch'], np.array(metric_values[self.training_context['metrics'].last_aggregate_idx:]).mean())
+                        self.epoch_metric_history.collect(k, self.training_context['current_epoch'], np.asarray(metric_values[self.training_context['metrics'].last_aggregate_idx:]).mean())
                         self.training_context['metrics'].last_aggregate_idx = len(metric_values)
 
 
@@ -977,8 +1004,8 @@ class ModelBase(object):
     def test(self, input,target):
         raise NotImplementedError
 
-    def trigger_when(self, when='on_batch_end',epoch=None,batch=None,action=None):
-        new_callbacks=LambdaCallback(when,epoch,batch,action)
+    def trigger_when(self, when='on_batch_end',epoch=None,batch=None,epoch_frequency=None,batch_frequency=None,action=None):
+        new_callbacks=LambdaCallback(when,epoch=epoch,batch=batch,epoch_frequency=epoch_frequency,batch_frequency=batch_frequency,function=action)
         self.with_callbacks(new_callbacks)
         return self
 
@@ -987,10 +1014,18 @@ class ModelBase(object):
         return self
 
     def cpu(self):
-       return NotImplemented
+       if self._model is not None and isinstance(self._model,Layer):
+           set_device('cpu')
+       elif self._model is not None and isinstance(self._model,Tensor):
+           self._model.cpu()
 
     def cuda(self):
-        return NotImplemented
+        if self._model is not None and isinstance(self._model, Layer):
+            set_device('cuda')
+        elif self._model is not None and isinstance(self._model, Tensor):
+            self._model.cuda()
+    def gpu(self):
+        self.cuda()
 
     #
     # def fit(self, x = None, y = None, batch_size = 8, epochs = 10,
@@ -1044,7 +1079,14 @@ class HistoryBase(OrderedDict):
         if data_name not in self:
             self.regist(data_name)
         if is_tensor(value):
-            self[data_name].append((step, to_numpy(value)))
+            value=value.clone().cpu().detach()
+            if ndim(value)==0:
+                value=value.item()
+            elif ndim(value)==1 and len(value)==1:
+                value = value[0].item()
+            else:
+                value=value.mean().item()
+            self[data_name].append((step, value))
         else:
             self[data_name].append((step, value))
 
@@ -1052,11 +1094,13 @@ class HistoryBase(OrderedDict):
     def reset(self):
         for i in range(len(self)):
             self.value_list[i]=[]
+    def get_keys(self):
+        return self.key_list
 
     def get_series(self,data_name):
         if data_name in self:
             steps,values=zip(*self[data_name].copy())
-            return steps,values
+            return list(steps),list(values)
         else:
             raise ValueError('{0} is not in this History.'.format(data_name))
 
