@@ -731,8 +731,9 @@ class AvgPool3d(_PoolNd):
         self.count_include_pad = kwargs.get('count_include_pad', False)
         self.divisor_override = kwargs.get('divisor_override', None)
 
-    def forward(self, input):
-        return F.avg_pool3d(input, self.kernel_size, self.strides, self.padding, self.ceil_mode, self.count_include_pad,
+    def forward(self, *x):
+        x = enforce_singleton(x)
+        return F.avg_pool3d(x, self.kernel_size, self.strides, self.padding, self.ceil_mode, self.count_include_pad,
                             self.divisor_override)
 
     def __setstate__(self, d):
@@ -744,20 +745,19 @@ class AvgPool3d(_PoolNd):
 
 class GlobalAvgPool2d(Layer):
     """Global Average Pooling Imprementation """
-    def __init__(self, keepdim=False, name='avg_pool'):
+    def __init__(self, keepdims=False, name='global_avg_pool',**kwargs):
         """
 
         Args:
-            keepdim ():
+            keepdims ():
             name ():
         """
-        super(GlobalAvgPool2d, self).__init__()
-        self.keepdim = keepdim
-        self.name = name
+        super(GlobalAvgPool2d, self).__init__(name=name)
+        self.keepdims = kwargs.get('keepdim',keepdims)
 
     def build(self, input_shape):
         if self._built == False:
-            if self.keepdim == True:
+            if self.keepdims == True:
                 output_shape = input_shape.clone()
                 output_shape[1] = 1
                 output_shape[2] = 1
@@ -769,7 +769,7 @@ class GlobalAvgPool2d(Layer):
     def forward(self, *x):
         x = enforce_singleton(x)
         N,C,H,W=x.size()
-        x = x.view(N, C, -1).mean(dim=-1, keepdim=self.keepdim)
+        x = x.view(N, C, -1).mean(dim=-1, keepdim=self.keepdims)
         return x
 
 
