@@ -39,7 +39,7 @@ __all__ = [ 'yolo4_body', 'YoloDetectionModel', 'DarknetConv2D', 'DarknetConv2D_
            'DarknetConv2D_BN_Leaky', 'YoloLayer']
 
 _session = get_session()
-_device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
+_device =get_device()
 _epsilon = _session.epsilon
 _trident_dir = _session.trident_dir
 
@@ -64,8 +64,6 @@ def DarknetConv2D_BN_Leaky(*args, **kwargs):
     darknet_conv_kwargs.update(kwargs)
     return Conv2d_Block(*args, **darknet_conv_kwargs)
 
-
-
 def DarknetConv2D_BN_Mish(*args, **kwargs):
     """Darknet Convolution2D followed by BatchNormalization and LeakyReLU."""
     darknet_conv_kwargs = {'use_bias': False, 'normalization':BatchNorm2d(momentum=0.03,eps=1e-4), 'activation': Mish}
@@ -311,7 +309,6 @@ class YoloLayer(Layer):
         #         conf_obj).item(),  #         "conf_noobj": to_cpu(conf_noobj).item(),  #         "grid_size": grid_size,  #     }  #  #     return output, total_loss
 
 
-
 class YoloDetectionModel(ImageDetectionModel):
     def __init__(self, inputs=None,  input_shape=None,output=None):
         super(YoloDetectionModel, self).__init__(inputs, input_shape,output)
