@@ -22,7 +22,7 @@ _session = get_session()
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
 __all__ = ['_ClassificationLoss', 'MSELoss', 'CrossEntropyLoss', 'NLLLoss', 'BCELoss', 'F1ScoreLoss', 'L1Loss', 'SmoothL1Loss', 'L2Loss', 'CosineSimilarityLoss',
            'ExponentialLoss', 'ItakuraSaitoLoss', 'MS_SSIMLoss', 'DiceLoss', 'WingLoss', 'AdaptiveWingLoss',
-           'IouLoss', 'FocalLoss', 'SoftIoULoss', 'CenterLoss', 'TripletLoss', 'TripletMarginLoss',
+           'IoULoss', 'FocalLoss', 'SoftIoULoss', 'CenterLoss', 'TripletLoss', 'TripletMarginLoss',
            'LovaszSoftmax', 'PerceptionLoss', 'EdgeLoss', 'TransformInvariantLoss', 'get_loss']
 
 
@@ -75,6 +75,7 @@ class _ClassificationLoss(Loss):
         self.cutoff = cutoff
         self.num_classes = None
         self.label_smooth = label_smooth
+        self.reduction=reduction
         # initilize weight
 
     def flatten_check(self, output, target):
@@ -109,11 +110,11 @@ class _ClassificationLoss(Loss):
         #output,target=self.flatten_check(output,target)
 
         if self.sample_weight is None:
-            self.sample_weight = ones(self.num_classes, requires_grad=False)
+            self.sample_weight = ones(self.num_classes, requires_grad=False).to(get_device())
         elif len(self.sample_weight) != self.num_classes:
             raise ValueError('weight should be 1-D tensor and length equal to numbers of filters')
         elif self.sample_weight.requires_grad!=False or self.sample_weight.dtype!=output.dtype or self.sample_weight.device!=output.device:
-            self.sample_weight = to_tensor(self.sample_weight, requires_grad=False)
+            self.sample_weight = to_tensor(self.sample_weight, requires_grad=False).to(get_device())
         else:
             pass
 
@@ -132,7 +133,7 @@ class _ClassificationLoss(Loss):
             self.is_logsoftmax = False
             self.from_logits = False
 
-        self.ignore_index_weight=ones_like(self.sample_weight,requires_grad=False,dtype=output.dtype).to(output.device)
+        self.ignore_index_weight=ones_like(self.sample_weight,requires_grad=False,dtype=output.dtype).to(get_device())
         # ignore_index
         with torch.no_grad():
             if isinstance(self.ignore_index, int) and 0 <= self.ignore_index < self.num_classes:
@@ -151,7 +152,7 @@ class _ClassificationLoss(Loss):
 
         # need target onehot but currently not
         if  target.dtype==torch.long and self.need_target_onehot == True and self.is_target_onehot == False:
-            target = make_onehot(target, num_classes=self.num_classes, axis=self.axis)
+            target = make_onehot(target, num_classes=self.num_classes, axis=self.axis).to(get_device())
             target.require_grads=False
             self.is_target_onehot=True
             if self.label_smooth:
@@ -372,13 +373,13 @@ class CrossEntropyLoss(_ClassificationLoss):
     >>> print(target.shape)
     torch.Size([4])
     >>> CrossEntropyLoss(reduction='mean')(output,target).cpu()
-    tensor(1.1034)
+    tensor(1.1305)
     >>> CrossEntropyLoss(reduction='sum')(output,target).cpu()
-    tensor(4.4136)
+    tensor(4.5221)
     >>> CrossEntropyLoss(label_smooth=True,reduction='mean')(output,target).cpu()
-    tensor(0.3453)
-    >>> CrossEntropyLoss(loss_weights=to_tensor([1.0,1.0,0]).float(),reduction='mean')(output,target).cpu()
-    tensor(0.8259)
+    tensor(1.0786)
+    >>> CrossEntropyLoss(sample_weight=to_tensor([1.0,1.0,0.5]).float(),reduction='mean')(output,target).cpu()
+    tensor(0.9889)
     >>> CrossEntropyLoss(ignore_index=2,reduction='mean')(output,target).cpu()
     tensor(0.8259)
 
@@ -434,9 +435,9 @@ class CrossEntropyLoss(_ClassificationLoss):
                     sample_weight=self.sample_weight.view( *reshape_shape)*self.ignore_index_weight.view( *reshape_shape)
             #-sum([p[i] * log2(q[i]) for i in range(len(p))])
             if self.is_logsoftmax == False:
-                loss = -(target * F.log_softmax(output,dim=self.axis)*sample_weight)
+                loss = -reduce_sum(target * F.log_softmax(output,dim=self.axis)*sample_weight,axis=1)
             else:
-                loss = -(target * output * sample_weight)
+                loss = -reduce_sum(target * output * sample_weight,axis=1)
         return loss
 
 
@@ -535,7 +536,7 @@ class NLLLoss(_ClassificationLoss):
     tensor(-1.3500)
     >>> NLLLoss(label_smooth=True,reduction='mean')(output,target).cpu()
     tensor(1.1034)
-    >>> NLLLoss(loss_weights=to_tensor([1.0,1.0,0]).float(),reduction='mean')(output,target).cpu()
+    >>> NLLLoss(loss_weights=to_tensor([1.0,1.0,0.5]).float(),reduction='mean')(output,target).cpu()
     tensor(-0.2625)
     >>> NLLLoss(ignore_index=2,reduction='mean')(output,target).cpu()
     tensor(-0.2625)
@@ -565,7 +566,7 @@ class NLLLoss(_ClassificationLoss):
         if self.is_target_onehot and ndim(target) == ndim(output):
             if not self.is_logsoftmax:
                 output=log_softmax(output,axis=self.axis)
-            loss=-target * output * loss_weights
+            loss=-reduce_sum(target * output * loss_weights,axis=1)
         else:
             loss = F.nll_loss(output, target, weight=self.sample_weight, ignore_index=self.ignore_index, reduction='none')
         return loss
@@ -854,7 +855,7 @@ class DiceLoss(_ClassificationLoss):
         axis=self.axis if self.axis>=0 else target.ndim+self.axis
         reduce_axes.remove(0)
         reduce_axes.remove(axis)
-        loss_weights=self.sample_weight.clone()
+        loss_weights=self.sample_weight.clone().to(get_device())
         # for k in range(target.ndim-self.loss_weights.ndim):
         #     loss_weights=loss_weights.expand_dims(0)
         intersection = reduce_sum(target * output, axis=reduce_axes)*loss_weights
@@ -1225,9 +1226,9 @@ class MS_SSIMLoss(_Loss):
         return 1 - msssim(output, target, window_size=self.window_size, normalize=True)
 
 
-class IouLoss(_Loss):
+class IoULoss(_Loss):
     def __init__(self, ignore_index=-1000, reduction='mean'):
-        super(IouLoss, self).__init__(reduction=reduction)
+        super(IoULoss, self).__init__(reduction=reduction)
         self.ignore_index = ignore_index
 
     def forward(self, output, target) -> 'loss':
